#[derive(Debug, Clone, Copy, PartialEq)]
pub enum MainAxisAlignment {
	/// Top or leading.
	///
	/// Children are aligned with the top or leading edge, without padding.
	Start,
	/// Children are centered, without padding.
	Center,
	/// Bottom or trailing.
	///
	/// Children are aligned with the bottom or trailing edge, without padding.
	End,
	/// Extra space is divided evenly between each child.
	SpaceBetween,
	/// Extra space is divided evenly between each child, as well as at the ends.
	SpaceEvenly,
	/// Space between each child, with less at the start and end.
	///
	/// This divides space such that each child is separated by `n` units,
	/// and the start and end have `n/2` units of padding.
	SpaceAround,
}
