use crate::{box_constraints::BoxConstraints, DataWrapper, Event, Point, Size};
use std::ops::DerefMut;

mod flex;
mod text;
mod widget_pod;
pub use flex::*;
pub use text::*;
pub use widget_pod::*;

pub trait Widget<T> {
	fn update(&mut self, data: &DataWrapper<T>);
	fn layout(&mut self, bc: &BoxConstraints) -> Size;
	fn paint(&mut self, buf: &mut [char], origin: Point, size: &Size);
	fn event(&mut self, data: &mut DataWrapper<T>, event: &Event);
}

impl<T> Widget<T> for Box<dyn Widget<T>> {
	fn update(&mut self, data: &DataWrapper<T>) {
		self.deref_mut().update(data)
	}
	fn layout(&mut self, bounds: &BoxConstraints) -> Size {
		self.deref_mut().layout(bounds)
	}
	fn paint(&mut self, buf: &mut [char], origin: Point, size: &Size) {
		self.deref_mut().paint(buf, origin, size)
	}
	fn event(&mut self, data: &mut DataWrapper<T>, event: &Event) {
		self.deref_mut().event(data, event)
	}
}
